#! /usr/bin/perl -w
use strict;
# This script allow to create the ELF and/or NCI picture of the reaction mechanism.
# To launch it: ./elf_nci_visualization.pl Input "name of the vmd input"
# Input format
# 1-First point Last point (ex: 1  3)
# 2-Image wanted NCI; ELF; BOTH (ELF and NCI)
# 3-Representation mode (CPK, line, licorice)
# 4-ELF isosurface value (ELF if 0 then iso = 0.8) or NCI
# (5-NCI isosurface value (NCI if 0 then iso = 0.2) NCI color scale (5 for -5.0 5.0))
# 6-Viewpoint

# Open input file and output files (vmd inputs)
if (!defined $ARGV[0]) {
    printf STDERR "no input file specified\n";
    printf STDERR "./elf_nci_visualization.pl Input\n";
    exit;
} else {
open INPUT, $ARGV[0] or die "Cannot Open $ARGV[0]:$!\n";
}

if (!defined $ARGV[1]) {
    printf STDERR "no vmd surface file specified\n";
    printf STDERR "vmd surface input = vmd_surface.vmd\n";
    open OUTPUT, ">vmd_surface.vmd" or die "Cannot Open vmd_surface.vmd:$!\n";
} else {
    open OUTPUT, ">$ARGV[1]" or die "Cannot Open $ARGV[1]:$!\n";
}

if (!defined $ARGV[2]) {
    printf STDERR "no vmd attractor file specified\n";
    printf STDERR "vmd attractor input = vmd_attractor.vmd\n";
    open OUTPUT2, ">vmd_attractor.vmd" or die "Cannot Open vmd_attractor.vmd:$!\n";
} else {
    open OUTPUT2, ">$ARGV[2]" or die "Cannot Open $ARGV[2]:$!\n";
}


# Define the variable
my $first=0;
my $last=0;
my @line_info;
my @line_info_2;
my @line_info_3;
my $isoelf;
my $isoelf_value;
my @line_info_elf;
my @line_info_nci;
my $isonci;
my $isonci_value;
my $scalencimin = 0;
my $scalencimax = 0;
my $view;
my @viewpoint;
my @line_info_nci;
my $type;
my $rep;

# reading all the line of the file
my @line = <INPUT>;
@line_info = split/\s+/, $line[0];
$first = $line_info[0];
$last = $line_info[1];
if ($first > $last || $first==0 || $last == 0) {
	printf STDERR "Problem with the first or last point check it!\n";
}

@line_info_2 = split/\s+/, $line[1];
$type = $line_info_2[0];

@line_info_3 = split/\s+/, $line[2];
$rep = $line_info_3[0];


###########################################################################
# Write surface vmd file
###########################################################################


# Set settings
printf OUTPUT "#!/usr/local/bin/vmd\n";
printf OUTPUT "# VMD script written by save_state \$Revision: 1.41\n";
printf OUTPUT "# VMD version: 1.8.6\n";
printf OUTPUT "#variable\n";
printf OUTPUT "for {set j $first} {\$j<=$last} {incr j} {\n";
printf OUTPUT " if {\$j<10} {\n";
printf OUTPUT "  set i 00\$j}\n";
printf OUTPUT " if {\$j>=10} {\n";
printf OUTPUT "  set i 0\$j}\n";
printf OUTPUT " if {\$j>=100} {\n";
printf OUTPUT "  set i \$j}\n";
printf OUTPUT " #vmd\n";
printf OUTPUT "  set viewplist {}\n";
printf OUTPUT "# Display settings\n";
printf OUTPUT "  display projection   Orthographic\n" ;
printf OUTPUT "  display nearclip set 0.000000\n" ;
printf OUTPUT "  color scale method BGR\n" ;
printf OUTPUT "  set colorcmds {\n" ;
printf OUTPUT "  {color Element {C} black}\n" ;
printf OUTPUT "  {color Element {Cl} green}\n" ;
printf OUTPUT "  {color Element {F} yellow2}\n" ;
printf OUTPUT "  {color Element {O} red}\n" ;
printf OUTPUT "  {color Element {N} blue}\n" ;
printf OUTPUT "  {color Element {H} white}\n";
printf OUTPUT "  }\n" ;
printf OUTPUT "  foreach colcmd \$colorcmds {\n" ;
printf OUTPUT "  set val [catch {eval \$colcmd}]\n" ;
printf OUTPUT "  }\n" ;
printf OUTPUT "   color Display {Background} white\n" ;
printf OUTPUT " \n";


#print STDERR "$type\n";

# Load inputs for ELF/NCI
printf OUTPUT " # load new molecule elf and/or nci\n";

if ($type eq "ELF"){
#	printf OUTPUT "  mol new geom\$\{i\}_esyn.cube         type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
	printf OUTPUT "  mol addfile geom\$\{i\}_elf.cube      type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
} 
elsif ($type eq "NCI"){
	printf OUTPUT "  mol addfile geom\$\{i\}-dens.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
	printf OUTPUT "  mol addfile geom\$\{i\}-grad.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
}  
elsif ($type eq "BOTH"){
#        printf OUTPUT "  mol new geom\$\{i\}_esyn.cube         type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
        printf OUTPUT "  mol addfile geom\$\{i\}_elf.cube      type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
        printf OUTPUT "  mol addfile geom\$\{i\}-dens.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
        printf OUTPUT "  mol addfile geom\$\{i\}-grad.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
} 
else {
	print STDERR " Revise function selection\n";
	print STDERR " Possibilities are ELF; NCI; BOTH\n";
	exit;
}


#Set representation
printf OUTPUT " #representation of the atoms\n";
printf OUTPUT "  mol delrep 0 top\n";
if ($rep eq "CPK") {
	printf OUTPUT "  mol representation CPK 0.500000 0.100000 118.000000 131.000000\n";
}
elsif ($rep eq "line") {
	printf OUTPUT "  mol representation Line\n";
}
elsif ($rep eq "licorice") {
	printf OUTPUT "  mol representation Licorice 0.100000 70.000000 70.000000\n";
}
else {
	print STDERR " Revise Representation selection\n";
	print STDERR " Possibilities are CPK, line, licorice\n";
	exit;
}

printf OUTPUT "  mol color Element\n";
printf OUTPUT "  mol selection {all}\n";
printf OUTPUT "  mol material Opaque\n";
printf OUTPUT "  mol addrep top\n";


#Set ELF and NCI isosurfaces
if ($type eq "ELF") {
	@line_info_elf = split/\s+/, $line[3];
        $isoelf_value = $line_info_elf[0];
} elsif ($type eq "NCI") {
        @line_info_nci = split/\s+/, $line[3];
        $isonci_value = $line_info_nci[0];
        $scalencimin = $line_info_nci[1];
	$scalencimax = $line_info_nci[2];
} elsif ($type eq "BOTH") {
	$isoelf_value = $line[3];
	@line_info_nci = split/\s+/, $line[4];
	$isonci_value = $line_info_nci[0];
        $scalencimin = $line_info_nci[1];
        $scalencimax = $line_info_nci[2];
}

if ($type eq "ELF" || $type eq "BOTH") {
	if ($isoelf_value == 0) {
		printf OUTPUT " # add representation of the isosurface elf\n";
		print STDERR " Attention no ELF isosurface cutoff defined, value chosen = 0.8\n";
		printf OUTPUT "  mol representation Isosurface 0.800000 0.000000 0.000000 0.000000 1 1\n";
		printf OUTPUT "  mol color ColorID 6\n";
		printf OUTPUT "  mol selection {all}\n"; 
		printf OUTPUT "  mol material Opaque\n"; 
		printf OUTPUT "  mol addrep top\n"; 
		printf OUTPUT "  mol selupdate 1 top 0\n"; 
		printf OUTPUT "  mol colupdate 1 top 0\n"; 
		printf OUTPUT "  mol scaleminmax top 1 0.000000 0.000000\n"; 
		printf OUTPUT "  mol smoothrep top 1 0\n"; 
		printf OUTPUT "  mol drawframes top 1 {now}\n"; 
	}
	elsif ($isoelf_value > 0) {
		printf OUTPUT " # add representation of the isosurface elf\n";
		printf OUTPUT "  mol representation Isosurface $isoelf_value 0.000000 0.000000 0.000000 1 1\n";
		printf OUTPUT "  mol color ColorID 6\n";
		printf OUTPUT "  mol selection {all}\n"; 
		printf OUTPUT "  mol material Opaque\n"; 
		printf OUTPUT "  mol addrep top\n"; 
		printf OUTPUT "  mol selupdate 1 top 0\n"; 
		printf OUTPUT "  mol colupdate 1 top 0\n"; 
		printf OUTPUT "  mol scaleminmax top 1 0.000000 0.000000\n"; 
		printf OUTPUT "  mol smoothrep top 1 0\n"; 
		printf OUTPUT "  mol drawframes top 1 {now}\n"; 
	}
	else {
		print STDERR " Error in the ELF isosurface value please revise it\n";
		print STDERR " Can be > 0 or = 0 (if 0, then the automatic value of 0.8 is taken\n";
		exit;
	}
	printf OUTPUT "  axes location off\n"; 
        printf OUTPUT "  display depthcue  off\n";
}


if ($type eq "BOTH" || $type eq "NCI") {
        if ($isonci_value == 0) {
		printf OUTPUT " # add representation of the isosurface nci\n";
                print STDERR " Attention no NCI isosurface cutoff defined, value chosen = 0.2\n";
                if ($type eq "NCI"){
                        printf OUTPUT "  mol representation Isosurface 0.200000 1.000000 0.000000 0.000000 1 1\n";
		        printf OUTPUT "  mol color Volume 0\n";
		        printf OUTPUT "  mol selection {all}\n"; 
		        printf OUTPUT "  mol material Opaque\n"; 
		        printf OUTPUT "  mol addrep top\n"; 
		        printf OUTPUT "  mol selupdate 1 top 0\n"; 
		        printf OUTPUT "  mol colupdate 1 top 0\n"; 
		        if ($scalencimin == 0 && $scalencimax == 0) {
			        print STDERR " Attention no NCI color scale defined, value chosen = -5.0; 5.0\n";
			        printf OUTPUT "  mol scaleminmax top 1 -5.000000 5.000000\n"; 
		        } elsif($scalencimin < $scalencimax) {
			        printf OUTPUT "  mol scaleminmax top 1 $scalencimin $scalencimax\n";
		        } else {
			        print STDERR " Error in the NCI colorscale value, please revise it\n";
			        print STDERR " Can be min < max or min = max = 0 (and then the automatic value of -5.0, 5.0 is taken\n";
			        exit;
		        }
		        printf OUTPUT "  mol smoothrep top 1 0\n"; 
		        printf OUTPUT "  mol drawframes top 1 {now}\n"; 
                }
                else {
                        printf OUTPUT "  mol representation Isosurface 0.200000 2.000000 0.000000 0.000000 1 1\n";
                        printf OUTPUT "  mol color Volume 1\n";
		        printf OUTPUT "  mol selection {all}\n"; 
		        printf OUTPUT "  mol material Opaque\n"; 
		        printf OUTPUT "  mol addrep top\n"; 
		        printf OUTPUT "  mol selupdate 2 top 0\n"; 
		        printf OUTPUT "  mol colupdate 2 top 0\n"; 
		        if ($scalencimin == 0 && $scalencimax == 0) {
			        print STDERR " Attention no NCI color scale defined, value chosen = -5.0; 5.0\n";
			        printf OUTPUT "  mol scaleminmax top 2 -5.000000 5.000000\n"; 
		        } elsif($scalencimin < $scalencimax) {
			        printf OUTPUT "  mol scaleminmax top 2 $scalencimin $scalencimax\n";
		        } else {
			        print STDERR " Error in the NCI colorscale value, please revise it\n";
			        print STDERR " Can be min < max or min = max = 0 (and then the automatic value of -5.0, 5.0 is taken\n";
			        exit;
		        }
		        printf OUTPUT "  mol smoothrep top 2 0\n"; 
		        printf OUTPUT "  mol drawframes top 2 {now}\n"; 
                }
		printf OUTPUT "  axes location off\n"; 
                printf OUTPUT "  display depthcue  off\n";
	        }
        elsif ($isonci_value > 0) {
		printf OUTPUT " # add representation of the isosurface nci\n";
                if ($type eq "NCI"){
                        printf OUTPUT "  mol representation Isosurface $isonci_value 1.000000 0.000000 0.000000 1 1\n";
		        printf OUTPUT "  mol color Volume 0\n";
		        printf OUTPUT "  mol selection {all}\n"; 
		        printf OUTPUT "  mol material Opaque\n"; 
		        printf OUTPUT "  mol addrep top\n"; 
		        printf OUTPUT "  mol selupdate 1 top 0\n"; 
		        printf OUTPUT "  mol colupdate 1 top 0\n"; 
		        if ($scalencimin == 0 && $scalencimax == 0) {
			        print STDERR " Attention no NCI color scale defined, value chosen = -5.0; 5.0\n";
			        printf OUTPUT "  mol scaleminmax top 1 -5.000000 5.000000\n"; 
		        } elsif($scalencimin < $scalencimax) {
			        printf OUTPUT "  mol scaleminmax top 1 $scalencimin $scalencimax\n";
		        } else {
			        print STDERR " Error in the NCI colorscale value, please revise it\n";
			        print STDERR " Can be min < max or min = max = 0 (and then the automatic value of -5.0, 5.0 is taken\n";
			        exit;
		        }
		        printf OUTPUT "  mol smoothrep top 1 0\n"; 
		        printf OUTPUT "  mol drawframes top 1 {now}\n"; 
                }
                else {
                        printf OUTPUT "  mol representation Isosurface $isonci_value 2.000000 0.000000 0.000000 1 1\n";
                        printf OUTPUT "  mol color Volume 1\n";
		        printf OUTPUT "  mol selection {all}\n"; 
		        printf OUTPUT "  mol material Opaque\n"; 
		        printf OUTPUT "  mol addrep top\n"; 
		        printf OUTPUT "  mol selupdate 2 top 0\n"; 
		        printf OUTPUT "  mol colupdate 2 top 0\n"; 
		        if ($scalencimin == 0 && $scalencimax == 0) {
			        print STDERR " Attention no NCI color scale defined, value chosen = -5.0; 5.0\n";
			        printf OUTPUT "  mol scaleminmax top 2 -5.000000 5.000000\n"; 
		        } elsif($scalencimin < $scalencimax) {
			        printf OUTPUT "  mol scaleminmax top 2 $scalencimin $scalencimax\n";
		        } else {
			        print STDERR " Error in the NCI colorscale value, please revise it\n";
			        print STDERR " Can be min < max or min = max = 0 (and then the automatic value of -5.0, 5.0 is taken\n";
			        exit;
		        }
		        printf OUTPUT "  mol smoothrep top 2 0\n"; 
		        printf OUTPUT "  mol drawframes top 2 {now}\n"; 
                }
		printf OUTPUT "  axes location off\n";
                printf OUTPUT "  display depthcue  off\n";
        }
        else {
                print STDERR " Error in the NCI isosurface value please revise it\n";
                print STDERR " Can be > 0 or = 0 (and then the automatic value of 0.2 is taken\n";
                exit;
	}
}


# Choose viewpoint
printf OUTPUT " #choose viewpoints\n";
if ($type eq "ELF" || $type eq "NCI") {
	$view = $line[4];
	@viewpoint = split/\s+/, $line[4];
} elsif ($type eq "BOTH") {
	$view = $line[5];
	@viewpoint = split/\s+/, $line[5];
}
if ($viewpoint[0] eq "set") {
	printf OUTPUT "  $view\n";
} else {
	print STDERR " Error in the viewpoint part, please check\n";
}

printf OUTPUT "  lappend viewplist [molinfo top]\n";
printf OUTPUT "  set topmol [molinfo top]\n";

printf OUTPUT "    # done with molecule 0\n";
printf OUTPUT "   foreach v \$viewplist {\n";
printf OUTPUT "     molinfo \$v set {center_matrix rotate_matrix scale_matrix global_matrix} \$viewpoints(\$v)\n";
printf OUTPUT "     }\n";
printf OUTPUT "   unset viewplist\n";


# Output image
printf OUTPUT " #output image to a file\n";
printf OUTPUT "  render snapshot geom\$\{i\}_elf_nci.tga\n";
printf OUTPUT "  mol delete all\n";
printf OUTPUT "}\n";

close OUTPUT;

#####################################################################
# Write attractor VMD file
#####################################################################


# Set settings
printf OUTPUT2 "#!/usr/local/bin/vmd\n";
printf OUTPUT2 "# VMD script written by save_state \$Revision: 1.41\n";
printf OUTPUT2 "# VMD version: 1.8.6\n";
printf OUTPUT2 "#variable\n";
printf OUTPUT2 "for {set j $first} {\$j<=$last} {incr j} {\n";
printf OUTPUT2 " if {\$j<10} {\n";
printf OUTPUT2 "  set i 00\$j}\n";
printf OUTPUT2 " if {\$j>=10} {\n";
printf OUTPUT2 "  set i 0\$j}\n";
printf OUTPUT2 " if {\$j>=100} {\n";
printf OUTPUT2 "  set i \$j}\n";
printf OUTPUT2 " #vmd\n";
printf OUTPUT2 "  set viewplist {}\n";
printf OUTPUT2 "# Display settings\n";
printf OUTPUT2 "  display projection   Orthographic\n" ;
printf OUTPUT2 "  display nearclip set 0.000000\n" ;
printf OUTPUT2 "  color scale method BGR\n" ;
printf OUTPUT2 "  set colorcmds {\n" ;
printf OUTPUT2 "  {color Element {C} black}\n" ;
printf OUTPUT2 "  {color Element {Cl} green}\n" ;
printf OUTPUT2 "  {color Element {F} yellow2}\n" ;
printf OUTPUT2 "  {color Element {O} red}\n" ;
printf OUTPUT2 "  {color Element {N} blue}\n" ;
printf OUTPUT2 "  {color Element {H} white}\n";
printf OUTPUT2 "  }\n" ;
printf OUTPUT2 "  foreach colcmd \$colorcmds {\n" ;
printf OUTPUT2 "  set val [catch {eval \$colcmd}]\n" ;
printf OUTPUT2 "  }\n" ;
printf OUTPUT2 "   color Display {Background} white\n" ;
printf OUTPUT2 " axes location off\n";
printf OUTPUT2 " \n";

# Load NCI Critical Point of Interaction
printf OUTPUT2 " # load new molecule\n";
printf OUTPUT2 " mol new geom\$\{i\}.wfn_topo.xyz         type xyz first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";

#Set representation
printf OUTPUT2 " #representation of the atoms\n";
printf OUTPUT2 "  mol delrep 0 top\n";
if ($rep eq "CPK") {
	printf OUTPUT2 "  mol representation CPK 0.500000 0.100000 118.000000 131.000000\n";
}
elsif ($rep eq "line") {
	printf OUTPUT2 "  mol representation Line\n";
}
elsif ($rep eq "licorice") {
	printf OUTPUT2 " mol representation Licorice 0.100000 70.000000 70.000000\n";
}
else {
	print STDERR " Revise Representation selection\n";
	print STDERR " Possibilities are CPK, line, licorice\n";
	exit;
}

printf OUTPUT2 "  mol color Element\n";
printf OUTPUT2 "  mol selection {all}\n";
printf OUTPUT2 "  mol material Opaque\n";
printf OUTPUT2 "  mol addrep top\n";
printf OUTPUT2 " # add representation of the interaction points\n"; 
printf OUTPUT2 "  mol representation CPK 0.200000 0.000000 118.000000 0.000000\n"; 
printf OUTPUT2 "  mol color Name\n"; 
printf OUTPUT2 "  mol selection {name X}\n"; 
printf OUTPUT2 "  mol material Opaque\n"; 
printf OUTPUT2 "  mol addrep top\n"; 

printf OUTPUT2 " # choose viewpoints\n";
printf OUTPUT2 " $line[5]\n";
printf OUTPUT2 "  lappend viewplist [molinfo top]\n";
printf OUTPUT2 "  set topmol [molinfo top]\n";
printf OUTPUT2 " # done with molecule 0\n";

printf OUTPUT2 " # load new molecule\n";
printf OUTPUT2 "  mol new geom\$\{i\}_ebas.xyz             type xyz first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
printf OUTPUT2 " # add representation of the attractors\n"; 
printf OUTPUT2 " mol delrep 0 top\n"; 
printf OUTPUT2 " mol representation CPK 0.200000 0.000000 118.000000 0.000000\n"; 
printf OUTPUT2 " mol color Element\n"; 
printf OUTPUT2 " mol selection {name xx}\n"; 
printf OUTPUT2 " mol material Opaque\n"; 
printf OUTPUT2 " mol addrep top\n"; 

printf OUTPUT2 " # choose viewpoints\n";
printf OUTPUT2 " $line[5]\n";
printf OUTPUT2 "  lappend viewplist [molinfo top]\n";
printf OUTPUT2 "  set topmol [molinfo top]\n";
printf OUTPUT2 " # done with molecule 1\n";

printf OUTPUT2 " foreach v \$viewplist {\n"; 
printf OUTPUT2 "    molinfo \$v set {center_matrix rotate_matrix scale_matrix global_matrix} \$viewpoints(\$v)\n"; 
printf OUTPUT2 "    }\n"; 
printf OUTPUT2 " unset viewplist\n"; 
printf OUTPUT2 " #output image to a file\n"; 
printf OUTPUT2 " render snapshot geom\$\{i\}_topo.tga\n"; 
printf OUTPUT2 " mol delete all\n"; 
printf OUTPUT2 "}\n"; 

close OUTPUT2;
